@@ -2,15 +2,16 @@
 -- must not be null.  The second arguement is the "compare string" and may
 -- or may not be null.  If the second string is null, the value returned is
 -- 3, otherwise it is the levenshtein difference between the two.
+-- Change 2010-10-18 Regina Obe - name verbose to var_verbose since get compile error in PostgreSQL 9.0
 CREATE OR REPLACE FUNCTION nullable_levenshtein(VARCHAR, VARCHAR) RETURNS INTEGER
 AS $_$
 DECLARE
   given_string VARCHAR;
   result INTEGER := 3;
-  verbose BOOLEAN := FALSE;
+  var_verbose BOOLEAN := FALSE; /**change from verbose to param_verbose since its a keyword and get compile error in 9.0 **/
 BEGIN
   IF $1 IS NULL THEN
-    IF verbose THEN
+    IF var_verbose THEN
       RAISE NOTICE 'nullable_levenshtein - given string is NULL!';
     END IF;
     RETURN NULL;
@@ -24,4 +25,4 @@ BEGIN
 
   RETURN result;
 END
-$_$ LANGUAGE plpgsql;
+$_$ LANGUAGE plpgsql IMMUTABLE COST 10;
