@@ -196,9 +196,9 @@ int lwpoint_is_empty(const LWPOINT *point);
 /*
 * Number of vertices?
 */
-uint32_t lwline_count_vertices(LWLINE *line);
-uint32_t lwpoly_count_vertices(LWPOLY *poly);
-uint32_t lwcollection_count_vertices(LWCOLLECTION *col);
+uint32_t lwline_count_vertices(const LWLINE *line);
+uint32_t lwpoly_count_vertices(const LWPOLY *poly);
+uint32_t lwcollection_count_vertices(const LWCOLLECTION *col);
 
 /*
 * DP simplification
@@ -262,6 +262,12 @@ int p2d_same(const POINT2D *p1, const POINT2D *p2);
 #define P3DM_SAME_STRICT(a,b) ((a)->x == (b)->x && (a)->y == (b)->y && (a)->m == (b)->m )
 #define P4D_SAME_STRICT(a,b) ((a)->x == (b)->x && (a)->y == (b)->y && (a)->z == (b)->z && (a)->m == (b)->m )
 
+/*
+* Projections
+*/
+int project_pt(const POINT2D *P, double distance, double azimuth, POINT2D *R);
+int project_pt_pt(const POINT4D *A, const POINT4D *B, double distance, POINT4D *R);
+
 /*
 * Area calculations
 */
@@ -365,7 +371,9 @@ int ptarray_isccw(const POINTARRAY *pa);
 /*
 * Same
 */
+char ptarray_same2d(const POINTARRAY *pa1, const POINTARRAY *pa2);
 char ptarray_same(const POINTARRAY *pa1, const POINTARRAY *pa2);
+char lwpoint_same2d(const LWPOINT *p1, const LWPOINT *p2);
 char lwpoint_same(const LWPOINT *p1, const LWPOINT *p2);
 char lwline_same(const LWLINE *p1, const LWLINE *p2);
 char lwpoly_same(const LWPOLY *p1, const LWPOLY *p2);
@@ -433,7 +441,6 @@ int lw_arc_side(const POINT2D *A1, const POINT2D *A2, const POINT2D *A3, const P
 int lw_arc_calculate_gbox_cartesian_2d(const POINT2D *A1, const POINT2D *A2, const POINT2D *A3, GBOX *gbox);
 double lw_arc_center(const POINT2D *p1, const POINT2D *p2, const POINT2D *p3, POINT2D *result);
 int lw_pt_in_seg(const POINT2D *P, const POINT2D *A1, const POINT2D *A2);
-int lw_pt_on_segment(const POINT2D* A1, const POINT2D* A2, const POINT2D* P);
 int lw_pt_in_arc(const POINT2D *P, const POINT2D *A1, const POINT2D *A2, const POINT2D *A3);
 int lw_arc_is_pt(const POINT2D *A1, const POINT2D *A2, const POINT2D *A3);
 double lw_seg_length(const POINT2D *A1, const POINT2D *A2);
@@ -443,8 +450,6 @@ int ptarray_contains_point(const POINTARRAY *pa, const POINT2D *pt);
 int ptarrayarc_contains_point(const POINTARRAY *pa, const POINT2D *pt);
 int ptarray_contains_point_partial(const POINTARRAY *pa, const POINT2D *pt, int check_closed, int *winding_number);
 int ptarrayarc_contains_point_partial(const POINTARRAY *pa, const POINT2D *pt, int check_closed, int *winding_number);
-int ptarray_raycast_intersections(const POINTARRAY *pa, const POINT2D *p, int *on_boundary);
-int ptarrayarc_raycast_intersections(const POINTARRAY *pa, const POINT2D *p, int *on_boundary);
 int lwcompound_contains_point(const LWCOMPOUND *comp, const POINT2D *pt);
 int lwgeom_contains_point(const LWGEOM *geom, const POINT2D *pt);
 
