@@ -28,6 +28,9 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include "../postgis_config.h"
+//#define POSTGIS_DEBUG_LEVEL 5
+
 #include "measures.h"
 #include "lwgeom_log.h"
 
@@ -299,7 +302,7 @@ lw_dist2d_recursive(const LWGEOM *lwg1, const LWGEOM *lwg2, DISTPTS *dl)
 			g1 = (LWGEOM *)lwg1;
 
 		if (lwgeom_is_empty(g1))
-			return LW_TRUE;
+			continue;
 
 		if (lw_dist2d_is_collection(g1))
 		{
@@ -329,10 +332,9 @@ lw_dist2d_recursive(const LWGEOM *lwg1, const LWGEOM *lwg2, DISTPTS *dl)
 			if (!g2->bbox)
 				lwgeom_add_bbox(g2);
 
-			/* If one of geometries is empty, return. True here only means continue searching. False would
-			 * have stopped the process*/
+			/* If one of geometries is empty, skip */
 			if (lwgeom_is_empty(g1) || lwgeom_is_empty(g2))
-				return LW_TRUE;
+				continue;
 
 			if ((dl->mode != DIST_MAX) && (!lw_dist2d_check_overlap(g1, g2)) &&
 			    (g1->type == LINETYPE || g1->type == POLYGONTYPE || g1->type == TRIANGLETYPE) &&
@@ -345,8 +347,10 @@ lw_dist2d_recursive(const LWGEOM *lwg1, const LWGEOM *lwg2, DISTPTS *dl)
 			{
 				if (!lw_dist2d_distribute_bruteforce(g1, g2, dl))
 					return LW_FALSE;
+				LWDEBUGF(2, "Distance so far: %.15g (%.15g tolerated)", dl->distance, dl->tolerance);
 				if (dl->distance <= dl->tolerance && dl->mode == DIST_MIN)
 					return LW_TRUE; /*just a check if the answer is already given*/
+				LWDEBUG(2, "Not below tolerance yet");
 			}
 		}
 	}
@@ -743,7 +747,7 @@ lw_dist2d_line_poly(LWLINE *line, LWPOLY *poly, DISTPTS *dl)
 	const POINT2D *pt = getPoint2d_cp(pa, 0);
 
 	/* Line has a pount outside poly. Check distance to outer ring only. */
-	if (ptarray_contains_point(poly->rings[0], pt) == LW_OUTSIDE)
+	if (ptarray_contains_point(poly->rings[0], pt) == LW_OUTSIDE || dl->mode == DIST_MAX)
 		return lw_dist2d_ptarray_ptarray(pa, poly->rings[0], dl);
 
 	for (uint32_t i = 1; i < poly->nrings; i++)
@@ -1134,9 +1138,13 @@ lw_dist2d_pt_ptarray(const POINT2D *p, POINTARRAY *pa, DISTPTS *dl)
 
 	start = getPoint2d_cp(pa, 0);
 
+	LWDEBUG(2, "lw_dist2d_pt_ptarray enter");
+
 	if (!lw_dist2d_pt_pt(p, start, dl))
 		return LW_FALSE;
 
+	LWDEBUGF(2, "lw_dist2d_pt_ptarray: distance from first point ? : %.15g", dl->distance);
+
 	for (uint32_t t = 1; t < pa->npoints; t++)
 	{
 		dl->twisted = twist;
@@ -1144,6 +1152,8 @@ lw_dist2d_pt_ptarray(const POINT2D *p, POINTARRAY *pa, DISTPTS *dl)
 		if (!lw_dist2d_pt_seg(p, start, end, dl))
 			return LW_FALSE;
 
+		LWDEBUGF(2, "lw_dist2d_pt_ptarray: distance from seg %lu ? : %.15g", t, dl->distance);
+
 		if (dl->distance <= dl->tolerance && dl->mode == DIST_MIN)
 			return LW_TRUE; /*just a check if the answer is already given*/
 		start = end;
@@ -2305,11 +2315,17 @@ To get this points it was necessary to change and it also showed to be about 10%
 int
 lw_dist2d_pt_seg(const POINT2D *p, const POINT2D *A, const POINT2D *B, DISTPTS *dl)
 {
-	POINT2D c;
 	double r;
+
+	LWDEBUG(2, "lw_dist2d_pt_seg called");
+
 	/*if start==end, then use pt distance */
 	if ((A->x == B->x) && (A->y == B->y))
+	{
+		LWDEBUG(2, "lw_dist2d_pt_seg found first and last segment points being the same");
 		return lw_dist2d_pt_pt(p, A, dl);
+	}
+
 
 	/*
 	 * otherwise, we use comp.graphics.algorithms
@@ -2329,6 +2345,8 @@ lw_dist2d_pt_seg(const POINT2D *p, const POINT2D *A, const POINT2D *B, DISTPTS *
 	r = ((p->x - A->x) * (B->x - A->x) + (p->y - A->y) * (B->y - A->y)) /
 	    ((B->x - A->x) * (B->x - A->x) + (B->y - A->y) * (B->y - A->y));
 
+	LWDEBUGF(2, "lw_dist2d_pt_seg found r = %.15g", r);
+
 	/*This is for finding the maxdistance.
 	the maxdistance have to be between two vertexes, compared to mindistance which can be between two vertexes.*/
 	if (dl->mode == DIST_MAX)
@@ -2352,12 +2370,41 @@ lw_dist2d_pt_seg(const POINT2D *p, const POINT2D *A, const POINT2D *B, DISTPTS *
 		dl->p2 = *p;
 	}
 
-	/*If the projection of point p on the segment is between A and B
-	then we find that "point on segment" and send it to lw_dist2d_pt_pt*/
-	c.x = A->x + r * (B->x - A->x);
-	c.y = A->y + r * (B->y - A->y);
 
-	return lw_dist2d_pt_pt(p, &c, dl);
+	/*
+		(2)
+				  (Ay-Cy)(Bx-Ax)-(Ax-Cx)(By-Ay)
+			s = -----------------------------
+                    L^2
+
+			Then the distance from C to P = |s|*L.
+	*/
+
+	double s = ((A->y - p->y) * (B->x - A->x) - (A->x - p->x) * (B->y - A->y)) /
+	           ((B->x - A->x) * (B->x - A->x) + (B->y - A->y) * (B->y - A->y));
+
+	double dist = fabs(s) * sqrt(((B->x - A->x) * (B->x - A->x) + (B->y - A->y) * (B->y - A->y)));
+	if ( dist < dl->distance )
+	{
+		dl->distance = dist;
+		{
+			POINT2D c;
+			c.x = A->x + r * (B->x - A->x);
+			c.y = A->y + r * (B->y - A->y);
+			if (dl->twisted > 0)
+			{
+				dl->p1 = *p;
+				dl->p2 = c;
+			}
+			else
+			{
+				dl->p1 = c;
+				dl->p2 = *p;
+			}
+		}
+	}
+
+	return LW_TRUE;
 }
 
 /** Compares incoming points and stores the points closest to each other or most far away from each other depending on
@@ -2487,7 +2534,7 @@ project_pt(const POINT2D *P, double distance, double azimuth, POINT2D *R)
 	double dx = cos(slope) * distance;
 	double dy = sin(slope) * distance;
 	R->x = P->x + dx;
-	R->y = P->x + dy;
+	R->y = P->y + dy;
 	return LW_TRUE;
 }
 
