@@ -27,12 +27,10 @@
 #include "liblwgeom.h"
 #include "pgsql_compat.h"
 
-#if POSTGIS_PGSQL_VERSION >= 160
+#if POSTGIS_PGSQL_VERSION > 150
 #include "varatt.h"
 #endif
 
-
-
 /****************************************************************************************/
 
 typedef enum
@@ -258,10 +256,6 @@ void lwpgerror(const char *fmt, ...);
 void lwpgnotice(const char *fmt, ...);
 void lwpgwarning(const char *fmt, ...);
 
-#if POSTGIS_PGSQL_VERSION < 100
-Datum CallerFInfoFunctionCall1(PGFunction func, FmgrInfo *flinfo, Oid collation, Datum arg1);
-Datum CallerFInfoFunctionCall2(PGFunction func, FmgrInfo *flinfo, Oid collation, Datum arg1, Datum arg2);
-#endif
 Datum CallerFInfoFunctionCall3(PGFunction func, FmgrInfo *flinfo, Oid collation, Datum arg1, Datum arg2, Datum arg3);
 
 
