@@ -15,6 +15,8 @@
 
 #include "../postgis_config.h"
 
+#include <math.h> /* for isnan */
+
 #include "shp2pgsql-core.h"
 #include "../liblwgeom/liblwgeom.h"
 #include "../liblwgeom/lwgeom_log.h" /* for LWDEBUG macros */
@@ -657,7 +659,7 @@ GeneratePolygonGeometry(SHPLOADERSTATE *state, SHPObject *obj, char **geometry)
 		polyring = Outer[pi];
 		while (polyring)
 		{
-			ring_total++;
+			ring_total = ring_total + 1;
 			polyring = polyring->next;
 		}
 
@@ -687,7 +689,7 @@ GeneratePolygonGeometry(SHPLOADERSTATE *state, SHPObject *obj, char **geometry)
 			lwpoly_add_ring(lwpoly, pa);
 
 			polyring = polyring->next;
-			ring_index++;
+			ring_index = ring_index + 1;
 		}
 
 		/* Generate the LWGEOM */
@@ -763,6 +765,7 @@ set_loader_config_defaults(SHPLOADERCONFIG *config)
 	config->quoteidentifiers = 0;
 	config->forceint4 = 0;
 	config->createindex = 0;
+	config->analyze = 1;
 	config->readshape = 1;
 	config->force_output = FORCE_OUTPUT_DISABLE;
 	config->encoding = strdup(ENCODING_DEFAULT);
@@ -841,7 +844,6 @@ ShpLoaderOpenShape(SHPLOADERSTATE *state)
 	SHPObject *obj = NULL;
 	int ret = SHPLOADEROK;
 	char name[MAXFIELDNAMELEN];
-	char name2[MAXFIELDNAMELEN];
 	DBFFieldType type = FTInvalid;
 	char *utf8str;
 
@@ -1161,23 +1163,20 @@ ShpLoaderOpenShape(SHPLOADERSTATE *state)
 		 * or after pgsql reserved attribute names
 		 */
 		if (name[0] == '_' ||
-		        ! strcmp(name, "gid") || ! strcmp(name, "tableoid") ||
-		        ! strcmp(name, "cmin") ||
-		        ! strcmp(name, "cmax") ||
-		        ! strcmp(name, "xmin") ||
-		        ! strcmp(name, "xmax") ||
-		        ! strcmp(name, "primary") ||
-		        ! strcmp(name, "oid") || ! strcmp(name, "ctid"))
+		    ! strcmp(name, "gid") ||
+		    ! strcmp(name, "tableoid") ||
+		    ! strcmp(name, "cmin") ||
+		    ! strcmp(name, "cmax") ||
+		    ! strcmp(name, "xmin") ||
+		    ! strcmp(name, "xmax") ||
+		    ! strcmp(name, "primary") ||
+		    ! strcmp(name, "oid") ||
+		    ! strcmp(name, "ctid"))
 		{
-			size_t len = strlen(name);
-			if (len > (MAXFIELDNAMELEN - 2))
-				len = MAXFIELDNAMELEN - 2;
-			strncpy(name2 + 2, name, len);
-			name2[MAXFIELDNAMELEN-1] = '\0';
-			name2[len + 2] = '\0';
-			name2[0] = '_';
-			name2[1] = '_';
-			strcpy(name, name2);
+			char tmp[MAXFIELDNAMELEN] = "__";
+			memcpy(tmp+2, name, MAXFIELDNAMELEN-2);
+			tmp[MAXFIELDNAMELEN-1] = '\0';
+			strncpy(name, tmp, MAXFIELDNAMELEN);
 		}
 
 		/* Avoid duplicating field names */
@@ -1926,13 +1925,17 @@ ShpLoaderGetSQLFooter(SHPLOADERSTATE *state, char **strfooter)
 		stringbuffer_aprintf(sb, "COMMIT;\n");
 	}
 
-	/* Always ANALYZE the resulting table, for better stats */
-	stringbuffer_aprintf(sb, "ANALYZE ");
-	if (state->config->schema)
+
+	if(state->config->analyze)
 	{
-		stringbuffer_aprintf(sb, "\"%s\".", state->config->schema);
+		/* Always ANALYZE the resulting table, for better stats */
+		stringbuffer_aprintf(sb, "ANALYZE ");
+		if (state->config->schema)
+		{
+			stringbuffer_aprintf(sb, "\"%s\".", state->config->schema);
+		}
+		stringbuffer_aprintf(sb, "\"%s\";\n", state->config->table);
 	}
-	stringbuffer_aprintf(sb, "\"%s\";\n", state->config->table);
 
 	/* Copy the string buffer into a new string, destroying the string buffer */
 	ret = (char *)malloc(strlen((char *)stringbuffer_getstring(sb)) + 1);
