@@ -116,6 +116,7 @@ dimensionality cases. (2D geometry) &&& (3D column), etc.
 #include "liblwgeom.h"
 #include "lwgeom_pg.h"       /* For debugging macros. */
 #include "gserialized_gist.h" /* For index common functions */
+#include "lwgeodetic.h"
 
 #include <math.h>
 #if HAVE_IEEEFP_H
@@ -349,11 +350,21 @@ cmp_int (const void *a, const void *b)
 * The difference between the fourth and first quintile values,
 * the "inter-quintile range"
 */
+// static int
+// range_quintile(int *vals, int nvals)
+// {
+// 	qsort(vals, nvals, sizeof(int), cmp_int);
+// 	return vals[4*nvals/5] - vals[nvals/5];
+// }
+
+/**
+* Lowest and highest bin values
+*/
 static int
-range_quintile(int *vals, int nvals)
+range_full(int *vals, int nvals)
 {
 	qsort(vals, nvals, sizeof(int), cmp_int);
-	return vals[4*nvals/5] - vals[nvals/5];
+	return vals[nvals-1] - vals[0];
 }
 
 /**
@@ -523,28 +534,28 @@ nd_stats_to_json(const ND_STATS *nd_stats)
 * Caller is responsible for freeing.
 * Currently only prints first two dimensions.
 */
-// static char*
-// nd_stats_to_grid(const ND_STATS *stats)
-// {
-//  char *rv;
-//  int j, k;
-//  int sizex = (int)roundf(stats->size[0]);
-//  int sizey = (int)roundf(stats->size[1]);
-//  stringbuffer_t *sb = stringbuffer_create();
-//
-//  for ( k = 0; k < sizey; k++ )
-//  {
-//      for ( j = 0; j < sizex; j++ )
-//      {
-//          stringbuffer_aprintf(sb, "%3d ", (int)roundf(stats->value[j + k*sizex]));
-//      }
-//      stringbuffer_append(sb,  "\n");
-//  }
-//
-//  rv = stringbuffer_getstringcopy(sb);
-//  stringbuffer_destroy(sb);
-//  return rv;
-// }
+static char*
+nd_stats_to_grid(const ND_STATS *stats)
+{
+ char *rv;
+ int j, k;
+ int sizex = (int)roundf(stats->size[0]);
+ int sizey = (int)roundf(stats->size[1]);
+ stringbuffer_t *sb = stringbuffer_create();
+
+ for ( k = 0; k < sizey; k++ )
+ {
+     for ( j = 0; j < sizex; j++ )
+     {
+         stringbuffer_aprintf(sb, "%3d ", (int)roundf(stats->value[j + k*sizex]));
+     }
+     stringbuffer_append(sb,  "\n");
+ }
+
+ rv = stringbuffer_getstringcopy(sb);
+ stringbuffer_destroy(sb);
+ return rv;
+}
 
 
 /** Expand the bounds of target to include source */
@@ -760,7 +771,8 @@ nd_box_ratio(const ND_BOX *b1, const ND_BOX *b2, int ndims)
 }
 
 /* How many bins shall we use in figuring out the distribution? */
-#define NUM_BINS 50
+#define MAX_NUM_BINS 50
+#define BIN_MIN_SIZE 10
 
 /**
 * Calculate how much a set of boxes is homogenously distributed
@@ -781,7 +793,7 @@ static int
 nd_box_array_distribution(const ND_BOX **nd_boxes, int num_boxes, const ND_BOX *extent, int ndims, double *distribution)
 {
 	int d, i, k, range;
-	int counts[NUM_BINS];
+	int *counts;
 	double smin, smax;   /* Spatial min, spatial max */
 	double swidth;       /* Spatial width of dimension */
 #if POSTGIS_DEBUG_LEVEL >= 3
@@ -790,11 +802,15 @@ nd_box_array_distribution(const ND_BOX **nd_boxes, int num_boxes, const ND_BOX *
 	int   bmin, bmax;   /* Bin min, bin max */
 	const ND_BOX *ndb;
 
+	int num_bins = Min(Max(2, num_boxes/BIN_MIN_SIZE), MAX_NUM_BINS);
+	counts = palloc0(num_bins * sizeof(int));
+
 	/* For each dimension... */
 	for ( d = 0; d < ndims; d++ )
 	{
 		/* Initialize counts for this dimension */
-		memset(counts, 0, sizeof(counts));
+		memset(counts, 0, num_bins * sizeof(int));
+
 
 		smin = extent->min[d];
 		smax = extent->max[d];
@@ -832,12 +848,12 @@ nd_box_array_distribution(const ND_BOX **nd_boxes, int num_boxes, const ND_BOX *
 			}
 
 			/* What bins does this range correspond to? */
-			bmin = floor(NUM_BINS * minoffset / swidth);
-			bmax = floor(NUM_BINS * maxoffset / swidth);
+			bmin = floor(num_bins * minoffset / swidth);
+			bmax = floor(num_bins * maxoffset / swidth);
 
 			/* Should only happen when maxoffset==swidth */
-			if (bmax >= NUM_BINS)
-				bmax = NUM_BINS-1;
+			if (bmax >= num_bins)
+				bmax = num_bins-1;
 
 			POSTGIS_DEBUGF(4, " dimension %d, feature %d: bin %d to bin %d", d, i, bmin, bmax);
 
@@ -850,11 +866,12 @@ nd_box_array_distribution(const ND_BOX **nd_boxes, int num_boxes, const ND_BOX *
 		}
 
 		/* How dispersed is the distribution of features across bins? */
-		range = range_quintile(counts, NUM_BINS);
+		// range = range_quintile(counts, num_bins);
+		range = range_full(counts, num_bins);
 
 #if POSTGIS_DEBUG_LEVEL >= 3
-		average = avg(counts, NUM_BINS);
-		sdev = stddev(counts, NUM_BINS);
+		average = avg(counts, num_bins);
+		sdev = stddev(counts, num_bins);
 		sdev_ratio = sdev/average;
 
 		POSTGIS_DEBUGF(3, " dimension %d: range = %d", d, range);
@@ -866,6 +883,8 @@ nd_box_array_distribution(const ND_BOX **nd_boxes, int num_boxes, const ND_BOX *
 		distribution[d] = range;
 	}
 
+	pfree(counts);
+
 	return true;
 }
 
@@ -908,28 +927,6 @@ pg_nd_stats_from_tuple(HeapTuple stats_tuple, int mode)
 	if ( mode == 2 ) stats_kind = STATISTIC_KIND_2D;
 
     /* Then read the geom status histogram from that */
-
-#if POSTGIS_PGSQL_VERSION < 100
-	{
-		float4 *floatptr;
-		int nvalues;
-
-		rv = get_attstatsslot(stats_tuple, 0, 0, stats_kind, InvalidOid,
-							NULL, NULL, NULL, &floatptr, &nvalues);
-
-		if ( ! rv ) {
-			POSTGIS_DEBUGF(2, "no slot of kind %d in stats tuple", stats_kind);
-			return NULL;
-		}
-
-		/* Clone the stats here so we can release the attstatsslot immediately */
-		nd_stats = palloc(sizeof(float) * nvalues);
-		memcpy(nd_stats, floatptr, sizeof(float) * nvalues);
-
-		/* Clean up */
-		free_attstatsslot(0, NULL, 0, floatptr, nvalues);
-	}
-#else /* PostgreSQL 10 or higher */
 	{
 		AttStatsSlot sslot;
 		rv = get_attstatsslot(&sslot, stats_tuple, stats_kind, InvalidOid,
@@ -945,8 +942,6 @@ pg_nd_stats_from_tuple(HeapTuple stats_tuple, int mode)
 
 		free_attstatsslot(&sslot);
 	}
-#endif
-
 	return nd_stats;
 }
 
@@ -1518,21 +1513,21 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 	}
 
 	/*
-	 * We'll build a histogram having stats->attr->attstattarget cells
-	 * on each side,  within reason... we'll use ndims*10000 as the
-	 * maximum number of cells.
+	 * We'll build a histogram having stats->attr->attstattarget
+	 * (default 100) cells on each side,  within reason...
+	 * we'll use ndims*100000 as the maximum number of cells.
 	 * Also, if we're sampling a relatively small table, we'll try to ensure that
-	 * we have an average of 5 features for each cell so the histogram isn't
-	 * so sparse.
+	 * we have a smaller grid.
 	 */
 #if POSTGIS_PGSQL_VERSION >= 170
 	histo_cells_target = (int)pow((double)(stats->attstattarget), (double)ndims);
+	POSTGIS_DEBUGF(3, " stats->attstattarget: %d", stats->attstattarget);
 #else
 	histo_cells_target = (int)pow((double)(stats->attr->attstattarget), (double)ndims);
-#endif
-	histo_cells_target = Min(histo_cells_target, ndims * 10000);
-	histo_cells_target = Min(histo_cells_target, (int)(total_rows/5));
 	POSTGIS_DEBUGF(3, " stats->attr->attstattarget: %d", stats->attr->attstattarget);
+#endif
+	histo_cells_target = Min(histo_cells_target, ndims * 100000);
+	histo_cells_target = Min(histo_cells_target, (int)(10 * ndims * total_rows));
 	POSTGIS_DEBUGF(3, " target # of histogram cells: %d", histo_cells_target);
 
 	/* If there's no useful features, we can't work out stats */
@@ -1978,6 +1973,8 @@ estimate_selectivity(const GBOX *box, const ND_STATS *nd_stats, int mode)
 	POSTGIS_DEBUGF(3, " nd_stats->extent: %s", nd_box_to_json(&(nd_stats->extent), nd_stats->ndims));
 	POSTGIS_DEBUGF(3, " nd_box: %s", nd_box_to_json(&(nd_box), gbox_ndims(box)));
 
+	// elog(DEBUG1, "out histogram:\n%s", nd_stats_to_grid(nd_stats));
+
 	/*
 	 * Search box completely misses histogram extent?
 	 * We have to intersect in all N dimensions or else we have
@@ -2082,10 +2079,12 @@ Datum _postgis_gserialized_stats(PG_FUNCTION_ARGS)
 		elog(ERROR, "stats for \"%s.%s\" do not exist", get_rel_name(table_oid), text_to_cstring(att_text));
 
 	/* Convert to JSON */
+	elog(DEBUG1, "stats grid:\n%s", nd_stats_to_grid(nd_stats));
 	str = nd_stats_to_json(nd_stats);
 	json = cstring_to_text(str);
 	pfree(str);
 	pfree(nd_stats);
+
 	PG_RETURN_TEXT_P(json);
 }
 
@@ -2268,7 +2267,7 @@ gserialized_sel_internal(PlannerInfo *root, List *args, int varRelid, int mode)
 	nd_stats = pg_nd_stats_from_tuple(vardata.statsTuple, mode);
 	ReleaseVariableStats(vardata);
 	selectivity = estimate_selectivity(&search_box, nd_stats, mode);
-	if (nd_stats) pfree(nd_stats);
+	pfree(nd_stats);
 	return selectivity;
 }
 
@@ -2286,6 +2285,18 @@ Datum gserialized_gist_sel(PG_FUNCTION_ARGS)
 }
 
 
+static Oid
+get_attype_by_name(Oid tbl, const char *colname)
+{
+	AttrNumber attnum = get_attnum(tbl, colname);
+	if (attnum == InvalidAttrNumber)
+		elog(ERROR, "cannot find column \"%s\" in table \"%s\"",
+			colname,
+			get_rel_name(tbl));
+
+	return get_atttype(tbl, attnum);
+}
+
 
 /**
  * Return the estimated extent of the table
@@ -2305,6 +2316,8 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 	bool only_parent = false;
 	int key_type, att_num;
 	size_t sz;
+	Oid atttypid;
+	bool is_geography = false;
 
 	/* We need to initialize the internal cache to access it later via postgis_oid() */
 	postgis_initialize_cache();
@@ -2348,8 +2361,11 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-	/* Read the extent from the head of the spatial index, if there is one */
+	atttypid = get_attype_by_name(tbl_oid, text_to_cstring(col));
+	if (atttypid == postgis_oid(GEOGRAPHYOID))
+		is_geography = true;
 
+	/* Read the extent from the head of the spatial index, if there is one */
 	idx_oid = table_get_spatial_index(tbl_oid, col, &key_type, &att_num);
 	if (idx_oid)
 	{
@@ -2365,7 +2381,7 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 		/* Fall back to reading the stats, if no index is found */
 
 		/* Estimated extent only returns 2D bounds, so use mode 2 */
-		nd_stats = pg_get_nd_stats_by_name(tbl_oid, col, 2, only_parent);
+		nd_stats = pg_get_nd_stats_by_name(tbl_oid, col, is_geography ? 3 : 2, only_parent);
 
 		/* Error out on no stats */
 		if ( ! nd_stats ) {
@@ -2374,17 +2390,32 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 		}
 
 		/* Construct the box */
-		gbox = palloc(sizeof(GBOX));
-		FLAGS_SET_GEODETIC(gbox->flags, 0);
-		FLAGS_SET_Z(gbox->flags, 0);
-		FLAGS_SET_M(gbox->flags, 0);
+		gbox = gbox_new(0);
 		gbox->xmin = nd_stats->extent.min[0];
 		gbox->xmax = nd_stats->extent.max[0];
 		gbox->ymin = nd_stats->extent.min[1];
 		gbox->ymax = nd_stats->extent.max[1];
+		if (is_geography)
+		{
+			FLAGS_SET_Z(gbox->flags, 1);
+			gbox->zmin = nd_stats->extent.min[2];
+			gbox->zmax = nd_stats->extent.max[2];
+		}
+
 		pfree(nd_stats);
 	}
 
+	/* Convert geocentric geography box into a planar box */
+	/* that users understand */
+	if (is_geography)
+	{
+		GBOX *gbox_planar = gbox_new(0);
+		gbox_geocentric_get_gbox_cartesian(gbox, gbox_planar);
+		PG_RETURN_POINTER(gbox_planar);
+	}
+	else
+		PG_RETURN_POINTER(gbox);
+
 	PG_RETURN_POINTER(gbox);
 }
 
@@ -2558,10 +2589,13 @@ spatial_index_read_extent(Oid idx_oid, int key_type, int att_num)
 	}
 	else if (key_type == STATISTIC_KIND_ND && bounds_gidx)
 	{
+		lwflags_t flags = 0;
 		if (gidx_is_unknown(bounds_gidx))
 			return NULL;
-		gbox = gbox_new(0);
-		gbox_from_gidx(bounds_gidx, gbox, 0);
+		FLAGS_SET_Z(flags, GIDX_NDIMS(bounds_gidx) > 2);
+		FLAGS_SET_M(flags, GIDX_NDIMS(bounds_gidx) > 3);
+		gbox = gbox_new(flags);
+		gbox_from_gidx(bounds_gidx, gbox, flags);
 	}
 	else
 		return NULL;
