@@ -105,6 +105,9 @@ Datum LWGEOM_setpoint_linestring(PG_FUNCTION_ARGS);
 Datum LWGEOM_asEWKT(PG_FUNCTION_ARGS);
 Datum LWGEOM_hasBBOX(PG_FUNCTION_ARGS);
 Datum LWGEOM_azimuth(PG_FUNCTION_ARGS);
+Datum geometry_project_direction(PG_FUNCTION_ARGS);
+Datum geometry_project_geometry(PG_FUNCTION_ARGS);
+Datum geometry_line_extend(PG_FUNCTION_ARGS);
 Datum LWGEOM_angle(PG_FUNCTION_ARGS);
 Datum LWGEOM_affine(PG_FUNCTION_ARGS);
 Datum LWGEOM_longitude_shift(PG_FUNCTION_ARGS);
@@ -2327,11 +2330,6 @@ Datum LWGEOM_addpoint(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-	if (gserialized_is_empty(pglwg2))
-	{
-		PG_RETURN_POINTER(pglwg1);
-	}
-
 	line = lwgeom_as_lwline(lwgeom_from_gserialized(pglwg1));
 
 	if (PG_NARGS() <= 2)
@@ -2574,6 +2572,121 @@ Datum LWGEOM_azimuth(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT8(result);
 }
 
+
+/**
+ * Project a new point from a start point, direction and distance.
+ * ST_Project(geometry, distance, azimuth)
+ * Azimuth is measured in radians, clockwise from north.
+ * Distance is in SRID units.
+ * Geometry must be point.
+ */
+PG_FUNCTION_INFO_V1(geometry_project_direction);
+Datum geometry_project_direction(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *geom1, *geom2;
+	LWPOINT *lwpoint1, *lwpoint2;
+	LWGEOM *lwgeom1, *lwgeom2;
+	double distance, azimuth;
+
+	geom1 = PG_GETARG_GSERIALIZED_P(0);
+	distance = PG_GETARG_FLOAT8(1);
+	azimuth = PG_GETARG_FLOAT8(2);
+	lwgeom1 = lwgeom_from_gserialized(geom1);
+	lwpoint1 = lwgeom_as_lwpoint(lwgeom1);
+
+	if (!lwpoint1)
+		lwpgerror("Argument must be POINT geometry");
+
+	if (lwgeom_is_empty(lwgeom1))
+		PG_RETURN_NULL();
+
+	lwpoint2 = lwpoint_project(lwpoint1, distance, azimuth);
+	lwgeom2 = lwpoint_as_lwgeom(lwpoint2);
+	geom2 = geometry_serialize(lwgeom2);
+	PG_RETURN_POINTER(geom2);
+}
+
+
+/**
+ * Project a new point from a start point, direction and distance.
+ * ST_Project(geometry, distance, azimuth)
+ * Azimuth is measured in radians, clockwise from north.
+ * Distance is in SRID units.
+ * Geometry must be point.
+ */
+PG_FUNCTION_INFO_V1(geometry_project_geometry);
+Datum geometry_project_geometry(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *geom1, *geom2, *geom3;
+	LWPOINT *lwpoint1, *lwpoint2, *lwpoint3;
+	LWGEOM *lwgeom1, *lwgeom2, *lwgeom3;
+	double distance;
+
+	geom1 = PG_GETARG_GSERIALIZED_P(0);
+	geom2 = PG_GETARG_GSERIALIZED_P(1);
+	distance = PG_GETARG_FLOAT8(2);
+
+	lwgeom1 = lwgeom_from_gserialized(geom1);
+	lwpoint1 = lwgeom_as_lwpoint(lwgeom1);
+	lwgeom2 = lwgeom_from_gserialized(geom2);
+	lwpoint2 = lwgeom_as_lwpoint(lwgeom2);
+
+	if (!(lwpoint1 && lwpoint2))
+		lwpgerror("Arguments must be POINT geometries");
+
+	if (lwgeom_is_empty(lwgeom1) || lwgeom_is_empty(lwgeom2))
+		PG_RETURN_NULL();
+
+	if (lwpoint_same2d(lwpoint1, lwpoint2))
+		PG_RETURN_POINTER(geom2);
+
+	lwpoint3 = lwpoint_project_lwpoint(lwpoint1, lwpoint2, distance);
+	lwgeom3 = lwpoint_as_lwgeom(lwpoint3);
+	geom3 = geometry_serialize(lwgeom3);
+
+	PG_RETURN_POINTER(geom3);
+}
+
+
+
+/**
+ * Extend the ends of a line outwards from
+ * the end, the start, or both, a set positive distance.
+ * ST_LineExtent(linestring, distance_forward, distance_backward (default 0.0)
+ * Geometry must be linestring.
+ */
+PG_FUNCTION_INFO_V1(geometry_line_extend);
+Datum geometry_line_extend(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *geom1, *geom2;
+	LWLINE *lwline1, *lwline2;
+	LWGEOM *lwgeom1, *lwgeom2;
+	double distance_forward, distance_backward;
+
+	geom1 = PG_GETARG_GSERIALIZED_P(0);
+	distance_forward = PG_GETARG_FLOAT8(1);
+	distance_backward = PG_GETARG_FLOAT8(2);
+
+	lwgeom1 = lwgeom_from_gserialized(geom1);
+	lwline1 = lwgeom_as_lwline(lwgeom1);
+	if (!lwline1)
+		lwpgerror("Argument must be LINESTRING geometry");
+
+	if (lwline_is_empty(lwline1))
+		PG_RETURN_NULL();
+
+	if (lwline_length_2d(lwline1) <= 0.0)
+		PG_RETURN_POINTER(geom1);
+
+	lwline2 = lwline_extend(lwline1, distance_forward, distance_backward);
+	lwgeom2 = lwline_as_lwgeom(lwline2);
+	geom2 = geometry_serialize(lwgeom2);
+
+	PG_RETURN_POINTER(geom2);
+}
+
+
+
 /**
  * Compute the angle defined by 3 points or the angle between 2 vectors
  * defined by 4 points
