@@ -57,21 +57,4 @@ SELECT 'e1', topology.GetFaceByPoint('topo','POINT(1 5)', 0);
 -- Ask for a Point with a tollerance too high (2 or more faces)
 SELECT 'e2', topology.GetFaceByPoint('topo','POINT(6 13)', 1);
 
--- Empty edge geometries should not make the function choke
--- See https://trac.osgeo.org/postgis/ticket/5946
-DO $$
-BEGIN
-  -- 1. corrupt topology
-  UPDATE topo.edge_data SET geom = 'LINESTRING EMPTY';
-  -- 2. Try to get a face by a point
-  BEGIN
-    SELECT 't5946', topology.GetFaceByPoint('topo','POINT(6 13)', 0);
-  EXCEPTION
-  WHEN OTHERS THEN
-    -- Strip details, we only want the first part
-    RAISE EXCEPTION '%', regexp_replace(SQLERRM, E'([^:]*): .*', E'\\1 (see #5946)');
-  END;
-END;
-$$ LANGUAGE 'plpgsql';
-
 SELECT NULL FROM topology.DropTopology('topo');
